/*
* stratus.h
*
* Copyright (C) 2019, Tristan Dagobert, CMLA, École Normale Supérieure Paris-Saclay.
*
* This software is a computer program.
* It is a part of a code which estimates, given a satellite time series
* the visibility of each image. Given a gray level time series, it provides a
* Boolean times series indicating the visibility (mainly the opaque clouds,
* haze and shadow).
*
*
* This software is governed by the CeCILL-C license under French law and
* abiding by the rules of distribution of free software.  You can  use,
* modify and/ or redistribute the software under the terms of the CeCILL-C
* license as circulated by CEA, CNRS and INRIA at the following URL
* "http://www.cecill.info".
*
* As a counterpart to the access to the source code and  rights to copy,
* modify and redistribute granted by the license, users are provided only
* with a limited warranty  and the software's author,  the holder of the
* economic rights,  and the successive licensors  have only  limited
* liability.
*
* In this respect, the user's attention is drawn to the risks associated
* with loading,  using,  modifying and/or developing or reproducing the
* software by the user in light of its specific status of free software,
* that may mean  that it is complicated to manipulate,  and  that  also
* therefore means  that it is reserved for developers  and  experienced
* professionals having in-depth computer knowledge. Users are therefore
* encouraged to load and test the software's suitability as regards their
* requirements in conditions enabling the security of their systems and/or
* data to be ensured and,  more generally, to use and operate it in the
* same conditions as regards security.
*
* The fact that you are presently reading this means that you have had
* knowledge of the CeCILL-C license and that you accept its terms.
*
*/

#if !(defined STRATUS_H)
#define STRATUS_H
/*============================================================================*/
#define IS_GROUND 0
#define IS_CLOUD  255
#define IS_CLEAR 0
#define IS_OCCULTED 255
/*============================================================================*/
struct options_s
{
    float alpha;
    float theta;
};
typedef struct options_s options_t;
/*============================================================================*/
#define PREFIXE     0
#define OUTPUT_ALL  1
#define SIFT        2

typedef struct criterion_s criterion_t;
struct criterion_s
{
    int type;
    int nb_ims;
    int nrow;
    int ncol;
    char ** filenames;
    float ** ims;
    float ** maps;
    criterion_t * next;
};

struct criteria_s
{
    int nb_criteria;
    int nrow;
    int ncol;
    int nb_ims;
    criterion_t * c;
};

typedef struct criteria_s criteria_t;
/*============================================================================*/
struct mappes_s
{
    float ** sift;
};
typedef struct mappes_s mappes_t;
/*============================================================================*/
criterion_t *
get_criterion(criterion_t * cn, int type);

mappes_t
compute_cloud_maps (criterion_t * cn, options_t options);

#endif
